import {
    Directive,
    ElementRef,
    HostListener,
    Input,
    OnChanges,
    SimpleChanges,
} from '@angular/core';

@Directive({
    selector: '[inputNumberValidator]',
})
export class InputNumberDirective implements OnChanges {
    // allow decimal numbers with up to 2 decimal places
    private decimalRegex: RegExp = new RegExp(/^-?[0-9]+(\.[0-9]*){0,1}$/g);
    // Allow key codes for special events. Reflect :
    // Backspace, tab, end, home
    private specialKeys: Array<string> = ['Backspace', 'Tab', 'Escape', 'End', 'Enter', 'Home', 'ArrowLeft', 'ArrowRight', 'Del', 'Delete', 'Clear', 'Copy', 'Paste', '-', '.'];
    @Input() limitDecimalPlaces = 0;
    @Input() disableCopy = false;
    @Input() disablePaste = false;
    @Input() allowNegative = false;
    @Input() allowDecimal = false;
    @Input() min = -Infinity;
    @Input() max = Infinity;
    @Input() pattern?: string | RegExp;
    inputElement: HTMLInputElement;
    valueSelected: boolean = false;

    constructor(public el: ElementRef) {
        this.inputElement = el.nativeElement;
    }

    /**
     * If the min or max input changes, then set the min or max property to the new value, or to -Infinity
     * or Infinity if the new value is not a number
     * @param {SimpleChanges} changes - SimpleChanges
     */
    ngOnChanges(changes: SimpleChanges): void {
        if (changes['min']) {
            this.min = isNaN(Number(this.min)) ? -Infinity : Number(this.min);
        }

        if (changes['max']) {
            this.max = isNaN(Number(this.max)) ? Infinity : Number(this.max);
        }
    }

    /* Preventing the user from entering anything other than a number. */
    @HostListener('beforeinput', ['$event'])
    onBeforeInput(event: InputEvent): any {
        if (isNaN(Number(event.data))) {            
            /* This is to allow the user to enter a decimal point. */
            if ((Boolean(this.allowDecimal) && event.data === '.') || (Boolean(this.allowNegative) && event.data === '-')) return;
            event.preventDefault();
            event.stopPropagation();
        }
    }

    /* This is to check if any text is selected. */
    @HostListener('select')
    onSelect() {
        this.valueSelected = true;
    }

    /** This method will be called whenever the element loses focus. */
    @HostListener('blur')
    onBlur() {
        this.valueSelected = false;
    }

    /* Allowing the user to use the keyboard to navigate the input field. */
    @HostListener('keydown', ['$event'])
    onKeyDown(event: KeyboardEvent): any {
        /* This is to allow the user to use the keyboard to navigate the input field. */
        if (
            this.specialKeys.indexOf(event.key) > -1 ||
            ((event.key === 'a' || event.code === 'KeyA') && event.ctrlKey === true) ||
            ((event.key === 'c' || event.code === 'KeyC') && event.ctrlKey === true) ||
            ((event.key === 'v' || event.code === 'KeyV') && event.ctrlKey === true) ||
            ((event.key === 'x' || event.code === 'KeyX') && event.ctrlKey === true) ||
            ((event.key === 'a' || event.code === 'KeyA') && event.metaKey === true) ||
            ((event.key === 'c' || event.code === 'KeyC') && event.metaKey === true) ||
            ((event.key === 'v' || event.code === 'KeyV') && event.metaKey === true) ||
            ((event.key === 'x' || event.code === 'KeyX') && event.metaKey === true)
        ) {
            // let it happen, don't do anything
            return;
        }
        let current: string = this.inputElement.value;
        if (this.valueSelected) {
            current = '';
            this.valueSelected = false;
        }

        let next: string = current.concat(event.key);
        // Ensure that it is a number and stop the keypress
        if (event.key === ' ' || isNaN(Number(event.key))) {
            event.preventDefault();
            return;
        }
        /* This is to prevent the user from entering a number that is greater than the max or less than the min. */
        if (next && Boolean(this.limitDecimalPlaces) && !String(next).match(this.decimalRegex)) {
            event.preventDefault();
            return;
        }
        const newNumber = Number(next);
        /* Preventing the user from entering a number that is greater than the max or less than the min. */
        if (newNumber > this.max || newNumber < this.min) {
            event.preventDefault();
            return;
        }
    }

    /* Preventing the user from copying the input value. */
    @HostListener('copy', ['$event'])
    onCopy(event: any) {
        if (Boolean(this.disableCopy)){
            event.preventDefault(); 
            return;
        } 
    }

    /* Preventing the user from pasting the input value. */
    @HostListener('paste', ['$event'])
    onPaste(event: any) {
        if (Boolean(this.disablePaste)) {
            event.preventDefault();
            return;
        }
        else this.validateInput(event);
    }

    /* Limiting the number of decimal places. */
    @HostListener('input', ['$event'])
    onInput(event: any) {
        this.validateInput(event);
    }

    validateInput(event:KeyboardEvent | any) {
        const input = event.target;
        const value = input.value;

        /* This is to prevent the user from entering a number that is greater than the max or less than the
        min. */
        if (value > this.max || value < this.min || value == '.') {
            input.value = null;
            event.preventDefault();
            return;
        } 
        // If the value contains more decimal places than allowed, truncate it
        if(Boolean(this.allowDecimal) && value.split('.').length > 2){
            input.value = value.substring(0,value.length-1);
        }
        /* This is to prevent the user from entering more than the allowed number of decimal places. */
        if (Boolean(this.allowDecimal) && Boolean(this.limitDecimalPlaces) && value.indexOf('.') !== -1 && value.split('.')[1].length > this.limitDecimalPlaces) {
            input.value = value.substring(0, value.indexOf('.') + this.limitDecimalPlaces + 1);
        }
        /* This is to prevent the user from entering more than one negative sign. */
        if (Boolean(this.allowNegative) && value.indexOf('-') !== -1 && value.split('-').length !== 2) {
            input.value = value.substring(0, 1);
        }
    }
}